--[[
	Copyright (C) Udorn (Blackhand)
	
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.	
--]]

--[[
	Bid or buys a given list of auctions.
--]]
vendor.BidOrBuyTask = {}
vendor.BidOrBuyTask.prototype = {}
vendor.BidOrBuyTask.metatable = {__index = vendor.BidOrBuyTask.prototype}

local log = vendor.Debug:new("BidOrBuyTask")

--[[
	Callback for the sniper.
--]]
local function _AskToBuy(itemLink, itemName, count, bid, buyout, highBidder, self)
	log:Debug("_AskToBuy")
	for k,v in pairs(self.auctions) do
		if (not v.wasBought) then
			local itemLinkKey = vendor.Items:GetItemLinkKey(itemLink)
			if (v.itemLinkKey == itemLinkKey and v.count == count) then
				if (bid and bid > 0 and bid == v.minBid and not highBidder) then
					log:Debug("bid it")
					v.wasBought = true
					return true, false
				end
				if (buyout and buyout > 0 and buyout == v.buyout) then
					log:Debug("buyout it")
					v.wasBought = true
					return false, true
				end
			end
		end
	end
	return false, false
end

local function _ScanFinished(self)
	log:Debug("_ScanFinished")
	self.scanFinished = true
end

--[[ 
	Creates a new instance.
--]]
function vendor.BidOrBuyTask:new(auctions)
	local instance = setmetatable({}, self.metatable)
	instance.auctions = auctions
	return instance
end

--[[
	Run function of the task.
--]]
function vendor.BidOrBuyTask.prototype:Run()
	log:Debug("Run enter")
	self.isRunning = true
	vendor.Sniper:RegisterExclusiveSniper(_AskToBuy, self)
	-- currently only possible for the same auctions
	local prevKey = nil
	for k,v in pairs(self.auctions) do
		if (prevKey and prevKey ~= v.itemLinkKey) then
			assert(false)
		else
			local itemLink = vendor.Items:GetItemLink(v.itemLinkKey)
			vendor.Scanner:Scan(itemLink, true, _ScanFinished, self)
			while (not self.scanFinished) do
				coroutine.yield()
			end
		end
	end
	vendor.Sniper:RegisterExclusiveSniper(nil)
	log:Debug("Run exit")
end

--[[
	Cancels the task and leaves it as soon as possible. 
--]]
function vendor.BidOrBuyTask.prototype:Cancel()
	self.isCancelled = true
	self.isRunning = false
	vendor.Sniper:RegisterExclusiveSniper(nil)
end

--[[
	Returns whether the task was canecelled.
--]]
function vendor.BidOrBuyTask.prototype:IsCancelled()
	return self.isCancelled
end

--[[
	Will be called by the TaskQueue, if the task has failed with an 
	unexpected error.
--]]
function vendor.BidOrBuyTask.prototype:Failed()
	self:Cancel()
end
